#include <string>
#include <iostream>
#include <stdlib.h>  
#include "mapgen.h"
#include "../map.h"
#include "../node.h"
#include "../util.h"
#include "math.h"

#define MAPGEN_AIR Node::getNodeByName("dragonblocks:air")
#define MAPGEN_GRASS Node::getNodeByName("dragonblocks:grass")
#define MAPGEN_DIRT Node::getNodeByName("dragonblocks:dirt")
#define MAPGEN_STONE Node::getNodeByName("dragonblocks:stone")
#define MAPGEN_BEDROCK Node::getNodeByName("dragonblocks:bedrock")
#define MAPGEN_MESE Node::getNodeByName("dragonblocks:mese")
#define MAPGEN_LEAVES Node::getNodeByName("dragonblocks:leaves")
#define MAPGEN_WOOD Node::getNodeByName("dragonblocks:wood")
#define MAPGEN_WATER Node::getNodeByName("dragonblocks:water")
#define MAPGEN_SAND Node::getNodeByName("dragonblocks:sand")

using namespace std;
Mapgen::Mapgen(Map *map){
	//Air
	for(int x = 0; x < 50; x++){
		for(int y = 0; y < 50; y++){
			map->setNode(x,y,MAPGEN_AIR);
			}
		}
	//Bottom
	int bottom[50];
	bottom[0] = random(20,25);
	for(int x = 1; x < 50; x++){
		bottom[x] = bottom[x-1] + random(-1,1);
		}
	//Smooth	
	for(int x = 1; x < 49; x++){
		if(bottom[x] > bottom[x+1] && bottom[x] > bottom[x-1]) bottom[x]--;
		else if(bottom[x] < bottom[x+1] && bottom[x] < bottom[x-1])bottom[x]++;	
		}
	//Dirt with Grass
	for(int x = 0; x < 50; x++){
		map->setNode(x,bottom[x],MAPGEN_GRASS);
		}
	//Dirt 		
	for(int x = 0; x < 50; x++){
		for(int y = bottom[x]+1; y < bottom[x]+5; y++){
			map->setNode(x,y,MAPGEN_DIRT);
			}
		}
	//Stone
	for(int x = 0; x < 50; x++){
		for(int y = bottom[x]+5; y < 49; y++){
			map->setNode(x,y,MAPGEN_STONE);
			}
		}
	//Mese
	for(int x = 0; x < 50; x++){
		for(int y = bottom[x]+10; y < 49; y++){
			if((rand() % 100) == 0) map->setNode(x,y,MAPGEN_MESE);
			}
		}		
	//Bedrock
	for(int x = 0; x < 50; x++){
		map->setNode(x,49,MAPGEN_BEDROCK);
		if(random(0,1) == 0){
			map->setNode(x,48,MAPGEN_BEDROCK);
			if(random(0,2) == 0){
				map->setNode(x,47,MAPGEN_BEDROCK);
				}
			}
		}
	//Water
	int flatcount = 0;
	for(int x = 1; x < 50; x++){
		if(bottom[x] == bottom[x-1]) flatcount++;
		else if(flatcount > 7){
			int leftborder = rand() % 2;
			int rightborder = rand() % 2;
			for(int mx = x-flatcount-3; mx < x+2; mx++){
				for(int y = bottom[mx];y < bottom[mx]+5;y++){
					if(map->getNode(mx,y) == MAPGEN_STONE) break;
					if(map->getNode(mx,y) == MAPGEN_DIRT || map->getNode(mx,y) == MAPGEN_GRASS) map->setNode(mx,y,MAPGEN_SAND);
					}
				}
			for(int mx = x-flatcount+leftborder;mx < x-1-rightborder;mx++) map->setNode(mx,bottom[x-1],MAPGEN_WATER);
			for(int mx = x-flatcount+1+leftborder;mx < x-2-rightborder;mx++) map->setNode(mx,bottom[x-1]+1,MAPGEN_WATER);
			for(int mx = x-flatcount+2+leftborder; mx < x-3-rightborder;mx++) map->setNode(mx,bottom[x-1]+2,MAPGEN_WATER);
			flatcount = 0;
			}
		else flatcount = 0;
		}	
	//Tree
	int treecount = rand() % 5;
	int treepos[treecount];
	for(int i = 0; i < treecount; i++){
		while(true){
			bool cont = true;
			treepos[i] = rand() % 50;
			for(int j = 0; j < i; j++){
				if(abs(treepos[j]-treepos[i]) < 3) cont = false; 
				}
			if(!(map->getNode(treepos[i],bottom[treepos[i]]) == MAPGEN_GRASS)) cont = false; 	
			if(cont) break;	
			}
		int x = treepos[i];
		int y = bottom[x] - 1;	
		map->setNode(x,y,MAPGEN_WOOD);
		map->setNode(x,y-1,MAPGEN_WOOD);
		map->setNode(x,y-2,MAPGEN_LEAVES);
		map->setNode(x,y-3,MAPGEN_LEAVES);
		map->setNode(x,y-4,MAPGEN_LEAVES);
		map->setNode(x-1,y-2,MAPGEN_LEAVES);
		map->setNode(x-1,y-3,MAPGEN_LEAVES);
		map->setNode(x-1,y-4,MAPGEN_LEAVES);
		map->setNode(x+1,y-2,MAPGEN_LEAVES);
		map->setNode(x+1,y-3,MAPGEN_LEAVES);
		map->setNode(x+1,y-4,MAPGEN_LEAVES);	
		}
	}
